#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
import sys

from apache_atlas.utils import next_id
from apache_atlas.utils import non_null


class AtlasBase(dict):
    def __init__(self, attrs):
        pass

    def __getattr__(self, attr):
        return self.get(attr)

    def __setattr__(self, key, value):
        self.__setitem__(key, value)

    def __setitem__(self, key, value):
        super(AtlasBase, self).__setitem__(key, value)
        self.__dict__.update({key: value})

    def __delattr__(self, item):
        self.__delitem__(item)

    def __delitem__(self, key):
        super(AtlasBase, self).__delitem__(key)
        del self.__dict__[key]

    def __repr__(self):
        return json.dumps(self)

    def type_coerce_attrs(self):
        pass


class AtlasBaseModelObject(AtlasBase):
    def __init__(self, members):
        AtlasBase.__init__(self, members)

        self.guid = members.get('guid')

        if self.guid is None:
            self.guid = next_id()


class TimeBoundary(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.startTime = attrs.get('startTime')
        self.endTime = attrs.get('endTime')
        self.timeZone = attrs.get('timeZone')


class Plist(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.list = non_null(attrs.get('list'), [])
        self.startIndex = non_null(attrs.get('startIndex'), 0)
        self.pageSize = non_null(attrs.get('pageSize'), 0)
        self.totalCount = non_null(attrs.get('totalCount'), 0)
        self.sortBy = attrs.get('sortBy')
        self.sortType = attrs.get('sortType')


class SearchFilter(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.startIndex = non_null(attrs.get('startIndex'), 0)
        self.maxsize = non_null(attrs.get('maxsize'), sys.maxsize)
        self.getCount = non_null(attrs.get('getCount'), True)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.enums import EntityStatus
from apache_atlas.model.glossary import AtlasTermAssignmentHeader
from apache_atlas.model.misc import AtlasBase, next_id, Plist, TimeBoundary
from apache_atlas.utils import non_null, type_coerce, type_coerce_dict, type_coerce_dict_list, type_coerce_list


class AtlasStruct(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.typeName = attrs.get('typeName')
        self.attributes = attrs.get('attributes')

    def get_attribute(self, name):
        return self.attributes[name] if self.attributes is not None and name in self.attributes else None

    def set_attribute(self, name, value):
        if self.attributes is None:
            self.attributes = {}

        self.attributes[name] = value

    def remove_attribute(self, name):
        if name and self.attributes is not None and name in self.attributes:
            del self.attributes[name]


class AtlasEntity(AtlasStruct):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStruct.__init__(self, attrs)

        self.guid = attrs.get('guid')
        self.homeId = attrs.get('homeId')
        self.relationshipAttributes = attrs.get('relationshipAttributes')
        self.classifications = attrs.get('classifications')
        self.meanings = attrs.get('meanings')
        self.customAttributes = attrs.get('customAttributes')
        self.businessAttributes = attrs.get('businessAttributes')
        self.labels = attrs.get('labels')
        self.status = attrs.get('status')
        self.isIncomplete = attrs.get('isIncomplete')
        self.provenanceType = attrs.get('provenanceType')
        self.proxy = attrs.get('proxy')
        self.version = attrs.get('version')
        self.createdBy = attrs.get('createdBy')
        self.updatedBy = attrs.get('updatedBy')
        self.createTime = attrs.get('createTime')
        self.updateTime = attrs.get('updateTime')

        if self.guid is None:
            self.guid = next_id()

    def type_coerce_attrs(self):
        super(AtlasEntity, self).type_coerce_attrs()

        self.classifications = type_coerce_list(self.classifications, AtlasClassification)
        self.meanings = type_coerce_list(self.meanings, AtlasTermAssignmentHeader)

    def get_relationship_attribute(self, name):
        return self.relationshipAttributes[
            name] if self.relationshipAttributes is not None and name in self.relationshipAttributes else None

    def set_relationship_attribute(self, name, value):
        if self.relationshipAttributes is None:
            self.relationshipAttributes = {}

        self.relationshipAttributes[name] = value

    def remove_relationship_attribute(self, name):
        if name and self.relationshipAttributes is not None and name in self.relationshipAttributes:
            del self.relationshipAttributes[name]


class AtlasEntityExtInfo(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.referredEntities = attrs.get('referredEntities')

    def type_coerce_attrs(self):
        super(AtlasEntityExtInfo, self).type_coerce_attrs()

        self.referredEntities = type_coerce_dict(self.referredEntities, AtlasEntity)

    def get_referenced_entity(self, guid):
        return self.referredEntities[
            guid] if self.referredEntities is not None and guid in self.referredEntities else None

    def add_referenced_entity(self, entity):
        if self.referredEntities is None:
            self.referredEntities = {}

        self.referredEntities[entity.guid] = entity


class AtlasEntityWithExtInfo(AtlasEntityExtInfo):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasEntityExtInfo.__init__(self, attrs)

        self.entity = attrs.get('entity')

    def type_coerce_attrs(self):
        super(AtlasEntityWithExtInfo, self).type_coerce_attrs()

        self.entity = type_coerce(self.entity, AtlasEntity)


class AtlasEntitiesWithExtInfo(AtlasEntityExtInfo):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasEntityExtInfo.__init__(self, attrs)

        self.entities = attrs.get('entities')

    def type_coerce_attrs(self):
        super(AtlasEntitiesWithExtInfo, self).type_coerce_attrs()

        self.entities = type_coerce_list(self.entities, AtlasEntity)

    def add_entity(self, entity):
        if self.entities is None:
            self.entities = []

        self.entities.append(entity)


class AtlasEntityHeader(AtlasStruct):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStruct.__init__(self, attrs)

        self.guid = attrs.get('guid')
        self.status = non_null(attrs.get('status'), EntityStatus.ACTIVE.name)
        self.displayText = attrs.get('displayText')
        self.classificationNames = attrs.get('classificationNames')
        self.classifications = attrs.get('classifications')
        self.meaningNames = attrs.get('meaningNames')
        self.meanings = attrs.get('.meanings')
        self.isIncomplete = non_null(attrs.get('isIncomplete'), False)
        self.labels = attrs.get('labels')

        if self.guid is None:
            self.guid = next_id()

    def type_coerce_attrs(self):
        super(AtlasEntityHeader, self).type_coerce_attrs()

        self.classifications = type_coerce_list(self.classifications, AtlasClassification)
        self.meanings = type_coerce_list(self.meanings, AtlasTermAssignmentHeader)


class AtlasClassification(AtlasStruct):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStruct.__init__(self, attrs)

        self.entityGuid = attrs.get('entityGuid')
        self.entityStatus = non_null(attrs.get('entityStatus'), EntityStatus.ACTIVE.name)
        self.propagate = attrs.get('propagate')
        self.validityPeriods = attrs.get('validityPeriods')
        self.removePropagationsOnEntityDelete = attrs.get('removePropagationsOnEntityDelete')

    def type_coerce_attrs(self):
        super(AtlasClassification, self).type_coerce_attrs()

        self.validityPeriods = type_coerce_list(self.validityPeriods, TimeBoundary)


class AtlasObjectId(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.guid = attrs.get('guid')
        self.typeName = attrs.get('typeName')
        self.uniqueAttributes = attrs.get('uniqueAttributes')


class AtlasRelatedObjectId(AtlasObjectId):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasObjectId.__init__(self, attrs)

        self.entityStatus = attrs.get('entityStatus')
        self.displayText = attrs.get('displayText')
        self.relationshipType = attrs.get('relationshipType')
        self.relationshipGuid = attrs.get('relationshipGuid')
        self.relationshipStatus = attrs.get('relationshipStatus')
        self.relationshipAttributes = attrs.get('relationshipAttributes')

    def type_coerce_attrs(self):
        super(AtlasRelatedObjectId, self).type_coerce_attrs()

        self.relationshipAttributes = type_coerce(self.relationshipAttributes, AtlasStruct)


class AtlasClassifications(Plist):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        Plist.__init__(self, attrs)

    def type_coerce_attrs(self):
        super(AtlasClassifications, self).type_coerce_attrs()

        Plist.list = type_coerce_list(Plist.list, AtlasClassification)


class AtlasEntityHeaders(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.guidHeaderMap = attrs.get('guidHeaderMap')

    def type_coerce_attrs(self):
        super(AtlasEntityHeaders, self).type_coerce_attrs()

        self.guidHeaderMap = type_coerce_dict(self.guidHeaderMap, AtlasEntityHeader)


class EntityMutationResponse(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.mutatedEntities = attrs.get('mutatedEntities')
        self.guidAssignments = attrs.get('guidAssignments')

    def type_coerce_attrs(self):
        super(EntityMutationResponse, self).type_coerce_attrs()

        self.mutatedEntities = type_coerce_dict_list(self.mutatedEntities, AtlasEntityHeader)

    def get_assigned_guid(self, guid):
        return self.guidAssignments.get(guid) if self.guidAssignments else None


class EntityMutations(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.entity_mutations = attrs.get('entity_mutations')

    def type_coerce_attrs(self):
        super(EntityMutations, self).type_coerce_attrs()

        self.entity_mutations = type_coerce_list(self.entity_mutations, EntityMutation)


class EntityMutation(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.op = attrs.get('op')
        self.entity = attrs.get('entity')

    def type_coerce_attrs(self):
        super(EntityMutation, self).type_coerce_attrs()

        self.entity = type_coerce(self.entity, AtlasEntity)


class AtlasCheckStateRequest(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.entityGuids = attrs.get('entityGuids')
        self.entityTypes = attrs.get('entityTypes')
        self.fixIssues = attrs.get('fixIssues')


class AtlasCheckStateResult(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.entitiesScanned = attrs.get('entitiesScanned')
        self.entitiesOk = attrs.get('entitiesOk')
        self.entitiesFixed = attrs.get('entitiesFixed')
        self.entitiesPartiallyFixed = attrs.get('entitiesPartiallyFixed')
        self.entitiesNotFixed = attrs.get('entitiesNotFixed')
        self.state = attrs.get('state')
        self.entities = attrs.get('entities')

    def type_coerce_attrs(self):
        super(AtlasCheckStateResult, self).type_coerce_attrs()

        self.entities = type_coerce(self.entities, AtlasEntityState)


class AtlasEntityState(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.guid = attrs.get('guid')
        self.typeName = attrs.get('typeName')
        self.name = attrs.get('name')
        self.status = attrs.get('status')
        self.state = attrs.get('state')
        self.issues = attrs.get('issues')



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.discovery import (AtlasQuickSearchResult, AtlasSearchResult,
                                          AtlasSuggestionsResult, AtlasUserSavedSearch)
from apache_atlas.utils import API, BASE_URI, HTTPMethod, HTTPStatus

DEFAULT_LIMIT = 100
DEFAULT_OFFSET = 0


class DiscoveryClient:
    DISCOVERY_URI = BASE_URI + "v2/search"
    DSL_SEARCH_URI = DISCOVERY_URI + "/dsl"
    FULL_TEXT_SEARCH_URI = DISCOVERY_URI + "/fulltext"
    BASIC_SEARCH_URI = DISCOVERY_URI + "/basic"
    FACETED_SEARCH_URI = BASIC_SEARCH_URI
    SAVED_SEARCH_URI = DISCOVERY_URI + "/saved"
    QUICK_SEARCH_URI = DISCOVERY_URI + "/quick"

    DSL_SEARCH = API(DSL_SEARCH_URI, HTTPMethod.GET, HTTPStatus.OK)
    FULL_TEXT_SEARCH = API(FULL_TEXT_SEARCH_URI, HTTPMethod.GET, HTTPStatus.OK)
    BASIC_SEARCH = API(BASIC_SEARCH_URI, HTTPMethod.GET, HTTPStatus.OK)
    FACETED_SEARCH = API(FACETED_SEARCH_URI, HTTPMethod.POST, HTTPStatus.OK)
    ATTRIBUTE_SEARCH = API(DISCOVERY_URI + "/attribute", HTTPMethod.GET, HTTPStatus.OK)
    RELATIONSHIP_SEARCH = API(DISCOVERY_URI + "/relationship", HTTPMethod.GET, HTTPStatus.OK)
    QUICK_SEARCH_WITH_GET = API(QUICK_SEARCH_URI, HTTPMethod.GET, HTTPStatus.OK)
    QUICK_SEARCH_WITH_POST = API(QUICK_SEARCH_URI, HTTPMethod.POST, HTTPStatus.OK)
    GET_SUGGESTIONS = API(DISCOVERY_URI + "/suggestions", HTTPMethod.GET, HTTPStatus.OK)

    # Saved Search
    GET_SAVED_SEARCHES = API(SAVED_SEARCH_URI, HTTPMethod.GET, HTTPStatus.OK)
    GET_SAVED_SEARCH = API(SAVED_SEARCH_URI + "/{search_name}", HTTPMethod.GET, HTTPStatus.OK)
    ADD_SAVED_SEARCH = API(SAVED_SEARCH_URI, HTTPMethod.POST, HTTPStatus.OK)
    UPDATE_SAVED_SEARCH = API(SAVED_SEARCH_URI, HTTPMethod.PUT, HTTPStatus.OK)
    DELETE_SAVED_SEARCH = API(SAVED_SEARCH_URI + "/{guid}", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    EXECUTE_SAVED_SEARCH_BY_NAME = API(SAVED_SEARCH_URI + "/execute/{search_name}", HTTPMethod.GET, HTTPStatus.OK)
    EXECUTE_SAVED_SEARCH_BY_GUID = API(SAVED_SEARCH_URI + "/execute/guid/{search_guid}", HTTPMethod.GET, HTTPStatus.OK)

    QUERY = "query"
    LIMIT = "limit"
    OFFSET = "offset"
    STATUS = "Status"

    def __init__(self, client):
        self.client = client

    def dsl_search(self, query):
        query_params = {DiscoveryClient.QUERY: query}

        return self.client.call_api(DiscoveryClient.DSL_SEARCH, AtlasSearchResult, query_params)

    def dsl_search_with_params(self, query, limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {DiscoveryClient.QUERY: query, DiscoveryClient.LIMIT: limit, DiscoveryClient.OFFSET: offset}

        return self.client.call_api(DiscoveryClient.DSL_SEARCH, AtlasSearchResult, query_params)

    def full_text_search(self, query):
        query_params = {DiscoveryClient.QUERY: query}

        return self.client.call_api(DiscoveryClient.FULL_TEXT_SEARCH, AtlasSearchResult, query_params)

    def full_text_search_with_params(self, query, limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {DiscoveryClient.QUERY: query, DiscoveryClient.LIMIT: limit, DiscoveryClient.OFFSET: offset}

        return self.client.call_api(DiscoveryClient.FULL_TEXT_SEARCH, AtlasSearchResult, query_params)

    def basic_search(self, type_name, classification, query, exclude_deleted_entities,
                     sort_by_attribute=None, sort_order=None,
                     limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {"typeName": type_name, "classification": classification, DiscoveryClient.QUERY: query,
                        "excludeDeletedEntities": exclude_deleted_entities, DiscoveryClient.LIMIT: limit,
                        DiscoveryClient.OFFSET: offset, "sortBy": sort_by_attribute, "sortOrder": sort_order}

        return self.client.call_api(DiscoveryClient.BASIC_SEARCH, AtlasSearchResult, query_params)

    def faceted_search(self, search_parameters):
        return self.client.call_api(DiscoveryClient.FACETED_SEARCH, AtlasSearchResult, None, search_parameters)

    def attribute_search(self, type_name, attr_name, attr_value_prefix, limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {"attrName": attr_name, "attrValuePrefix": attr_value_prefix, "typeName": type_name,
                        DiscoveryClient.LIMIT: limit, DiscoveryClient.OFFSET: offset}

        return self.client.call_api(DiscoveryClient.ATTRIBUTE_SEARCH, AtlasSearchResult, query_params)

    def relationship_search(self, guid, relation, sort_by_attribute, sort_order, exclude_deleted_entities,
                            limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {"guid": guid, "relation": relation, "sortBy": sort_by_attribute,
                        "excludeDeletedEntities": exclude_deleted_entities,
                        DiscoveryClient.LIMIT: limit, DiscoveryClient.OFFSET: offset}

        if sort_order is not None:
            query_params["sortOrder"] = str(sort_order)

        return self.client.call_api(DiscoveryClient.RELATIONSHIP_SEARCH, AtlasSearchResult, query_params)

    def quick_search(self, query, type_name, exclude_deleted_entities, limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {"query": query, "typeName": type_name, "excludeDeletedEntities": exclude_deleted_entities,
                        DiscoveryClient.LIMIT: limit, DiscoveryClient.OFFSET: offset}

        return self.client.call_api(DiscoveryClient.QUICK_SEARCH_WITH_GET, AtlasQuickSearchResult, query_params)

    def quick_search_with_param(self, quick_search_parameters):
        return self.client.call_api(DiscoveryClient.QUICK_SEARCH_WITH_POST, AtlasQuickSearchResult, None,
                                    quick_search_parameters)

    # fieldName should be the parameter on which indexing is enabled such as "qualifiedName"
    def get_suggestions(self, prefix_string, field_name):
        query_params = {}

        if prefix_string is not None:
            query_params["prefixString"] = prefix_string

        if field_name is not None:
            query_params["fieldName"] = field_name

        return self.client.call_api(DiscoveryClient.GET_SUGGESTIONS, AtlasSuggestionsResult, query_params)

    def get_saved_searches(self, user_name):
        query_params = {"user": user_name}

        return self.client.call_api(DiscoveryClient.GET_SAVED_SEARCHES, list, query_params)

    def get_saved_search(self, user_name, search_name):
        query_params = {"user": user_name}

        return self.client.call_api(DiscoveryClient.GET_SAVED_SEARCH.format_path({'search_name': search_name}),
                                    AtlasUserSavedSearch, query_params)

    def add_saved_search(self, saved_search):
        return self.client.call_api(DiscoveryClient.ADD_SAVED_SEARCH, AtlasUserSavedSearch, None, saved_search)

    def update_saved_search(self, saved_search):
        return self.client.call_api(DiscoveryClient.UPDATE_SAVED_SEARCH, AtlasUserSavedSearch, None, saved_search)

    def delete_saved_search(self, guid):
        return self.client.call_api(DiscoveryClient.DELETE_SAVED_SEARCH.format_path({'guid': guid}))

    def execute_saved_search_with_user_name(self, user_name, search_name):
        query_params = {"user": user_name}

        return self.client.call_api(
            DiscoveryClient.EXECUTE_SAVED_SEARCH_BY_NAME.format_path({'search_name': search_name}),
            AtlasSearchResult, query_params)

    def execute_saved_search_with_search_guid(self, search_guid):
        return self.client.call_api(
            DiscoveryClient.EXECUTE_SAVED_SEARCH_BY_GUID.format_path({'search_guid': search_guid}),
            AtlasSearchResult)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.instance import AtlasClassifications
from apache_atlas.model.instance import AtlasEntitiesWithExtInfo
from apache_atlas.model.instance import AtlasEntityHeader
from apache_atlas.model.instance import AtlasEntityHeaders
from apache_atlas.model.instance import AtlasEntityWithExtInfo
from apache_atlas.model.instance import EntityMutationResponse
from apache_atlas.utils import API
from apache_atlas.utils import APPLICATION_JSON
from apache_atlas.utils import APPLICATION_OCTET_STREAM
from apache_atlas.utils import attributes_to_params
from apache_atlas.utils import BASE_URI
from apache_atlas.utils import HTTPMethod
from apache_atlas.utils import HTTPStatus
from apache_atlas.utils import list_attributes_to_params
from apache_atlas.utils import MULTIPART_FORM_DATA


class EntityClient:
    ENTITY_API = BASE_URI + "v2/entity/"
    PREFIX_ATTR = "attr:"
    PREFIX_ATTR_ = "attr_"
    ADMIN_API = BASE_URI + "admin/"
    ENTITY_PURGE_API = ADMIN_API + "purge/"
    ENTITY_BULK_API = ENTITY_API + "bulk/"
    BULK_SET_CLASSIFICATIONS = "bulk/setClassifications"
    BULK_HEADERS = "bulk/headers"

    # Entity APIs
    GET_ENTITY_BY_GUID = API(ENTITY_API + "guid", HTTPMethod.GET, HTTPStatus.OK)
    GET_ENTITY_BY_UNIQUE_ATTRIBUTE = API(ENTITY_API + "uniqueAttribute/type", HTTPMethod.GET, HTTPStatus.OK)
    GET_ENTITIES_BY_GUIDS = API(ENTITY_BULK_API, HTTPMethod.GET, HTTPStatus.OK)
    GET_ENTITIES_BY_UNIQUE_ATTRIBUTE = API(ENTITY_BULK_API + "uniqueAttribute/type", HTTPMethod.GET, HTTPStatus.OK)
    GET_ENTITY_HEADER_BY_GUID = API(ENTITY_API + "guid/{entity_guid}/header", HTTPMethod.GET, HTTPStatus.OK)
    GET_ENTITY_HEADER_BY_UNIQUE_ATTRIBUTE = API(
        ENTITY_API + "uniqueAttribute/type/{type_name}/header", HTTPMethod.GET, HTTPStatus.OK)

    GET_AUDIT_EVENTS = API(ENTITY_API + "{guid}/audit", HTTPMethod.GET, HTTPStatus.OK)
    CREATE_ENTITY = API(ENTITY_API, HTTPMethod.POST, HTTPStatus.OK)
    CREATE_ENTITIES = API(ENTITY_BULK_API, HTTPMethod.POST, HTTPStatus.OK)
    UPDATE_ENTITY = API(ENTITY_API, HTTPMethod.POST, HTTPStatus.OK)
    UPDATE_ENTITY_BY_ATTRIBUTE = API(ENTITY_API + "uniqueAttribute/type/", HTTPMethod.PUT, HTTPStatus.OK)
    UPDATE_ENTITIES = API(ENTITY_BULK_API, HTTPMethod.POST, HTTPStatus.OK)
    PARTIAL_UPDATE_ENTITY_BY_GUID = API(ENTITY_API + "guid/{entity_guid}", HTTPMethod.PUT, HTTPStatus.OK)
    DELETE_ENTITY_BY_GUID = API(ENTITY_API + "guid", HTTPMethod.DELETE, HTTPStatus.OK)
    DELETE_ENTITY_BY_ATTRIBUTE = API(ENTITY_API + "uniqueAttribute/type/", HTTPMethod.DELETE, HTTPStatus.OK)
    DELETE_ENTITIES_BY_GUIDS = API(ENTITY_BULK_API, HTTPMethod.DELETE, HTTPStatus.OK)
    PURGE_ENTITIES_BY_GUIDS = API(ENTITY_PURGE_API, HTTPMethod.PUT, HTTPStatus.OK)

    # Classification APIs
    GET_CLASSIFICATIONS = API(ENTITY_API + "guid/{guid}/classifications", HTTPMethod.GET, HTTPStatus.OK)
    GET_FROM_CLASSIFICATION = API(
        ENTITY_API + "guid/{entity_guid}/classification/{classification}", HTTPMethod.GET, HTTPStatus.OK)
    ADD_CLASSIFICATIONS = API(ENTITY_API + "guid/{guid}/classifications", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    ADD_CLASSIFICATION = API(ENTITY_BULK_API + "/classification", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    ADD_CLASSIFICATION_BY_TYPE_AND_ATTRIBUTE = API(
        ENTITY_API + "uniqueAttribute/type/{type_name}/classifications", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    UPDATE_CLASSIFICATIONS = API(ENTITY_API + "guid/{guid}/classifications", HTTPMethod.PUT, HTTPStatus.NO_CONTENT)
    UPDATE_CLASSIFICATION_BY_TYPE_AND_ATTRIBUTE = API(
        ENTITY_API + "uniqueAttribute/type/{type_name}/classifications", HTTPMethod.PUT, HTTPStatus.NO_CONTENT)
    UPDATE_BULK_SET_CLASSIFICATIONS = API(ENTITY_API + BULK_SET_CLASSIFICATIONS, HTTPMethod.POST, HTTPStatus.OK)
    DELETE_CLASSIFICATION = API(
        ENTITY_API + "guid/{guid}/classification/{classification_name}", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    DELETE_CLASSIFICATION_BY_TYPE_AND_ATTRIBUTE = API(
        ENTITY_API + "uniqueAttribute/type/{type_name}/classification/{" "classification_name}", HTTPMethod.DELETE,
        HTTPStatus.NO_CONTENT)
    GET_BULK_HEADERS = API(ENTITY_API + BULK_HEADERS, HTTPMethod.GET, HTTPStatus.OK)

    # Business Attributes APIs
    ADD_BUSINESS_ATTRIBUTE = API(
        ENTITY_API + "guid/{entity_guid}/businessmetadata", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    ADD_BUSINESS_ATTRIBUTE_BY_NAME = API(
        ENTITY_API + "guid/{entity_guid}/businessmetadata/{bm_name}", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    DELETE_BUSINESS_ATTRIBUTE = API(
        ENTITY_API + "guid/{entity_guid}/businessmetadata", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    DELETE_BUSINESS_ATTRIBUTE_BY_NAME = API(
        ENTITY_API + "guid/{entity_guid}/businessmetadata/{bm_name}", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    GET_BUSINESS_METADATA_TEMPLATE = API(ENTITY_API + "businessmetadata/import/template",
                                         HTTPMethod.GET, HTTPStatus.OK, APPLICATION_JSON, APPLICATION_OCTET_STREAM)
    IMPORT_BUSINESS_METADATA = API(ENTITY_API + "businessmetadata/import", HTTPMethod.POST,
                                   HTTPStatus.OK, MULTIPART_FORM_DATA, APPLICATION_JSON)

    # Labels APIs
    ADD_LABELS = API(ENTITY_API + "guid/{entity_guid}/labels", HTTPMethod.PUT, HTTPStatus.NO_CONTENT)
    ADD_LABELS_BY_UNIQUE_ATTRIBUTE = API(
        ENTITY_API + "uniqueAttribute/type/{type_name}/labels", HTTPMethod.PUT, HTTPStatus.NO_CONTENT)
    SET_LABELS = API(ENTITY_API + "guid/%s/labels", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    SET_LABELS_BY_UNIQUE_ATTRIBUTE = API(
        ENTITY_API + "uniqueAttribute/type/{entity_guid}/labels", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    DELETE_LABELS = API(ENTITY_API + "guid/{entity_guid}/labels", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    DELETE_LABELS_BY_UNIQUE_ATTRIBUTE = API(
        ENTITY_API + "uniqueAttribute/type/{type_name}/labels", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)

    def __init__(self, client):
        self.client = client

    def get_entity_by_guid(self, guid, min_ext_info=False, ignore_relationships=False):
        query_params = {"minExtInfo": min_ext_info, "ignoreRelationships": ignore_relationships}

        return self.client.call_api(EntityClient.GET_ENTITY_BY_GUID.format_path_with_params(guid),
                                    AtlasEntityWithExtInfo, query_params)

    def get_entity_by_attribute(self, type_name, uniq_attributes, min_ext_info=False, ignore_relationships=False):
        query_params = attributes_to_params(uniq_attributes)
        query_params["minExtInfo"] = min_ext_info
        query_params["ignoreRelationships"] = ignore_relationships

        return self.client.call_api(EntityClient.GET_ENTITY_BY_UNIQUE_ATTRIBUTE.format_path_with_params(type_name),
                                    AtlasEntityWithExtInfo, query_params)

    def get_entities_by_guids(self, guids, min_ext_info=False, ignore_relationships=False):
        query_params = {"guid": guids, "minExtInfo": min_ext_info, "ignoreRelationships": ignore_relationships}

        return self.client.call_api(EntityClient.GET_ENTITIES_BY_GUIDS, AtlasEntitiesWithExtInfo, query_params)

    def get_entities_by_attribute(self, type_name, uniq_attributes_list, min_ext_info=False,
                                  ignore_relationships=False):
        query_params = list_attributes_to_params(uniq_attributes_list)
        query_params["minExtInfo"] = min_ext_info
        query_params["ignoreRelationships"] = ignore_relationships

        return self.client.call_api(EntityClient.GET_ENTITIES_BY_UNIQUE_ATTRIBUTE.format_path_with_params(type_name),
                                    AtlasEntitiesWithExtInfo, query_params)

    def get_entity_header_by_guid(self, entity_guid):
        return self.client.call_api(EntityClient.GET_ENTITY_HEADER_BY_GUID.format_path({'entity_guid': entity_guid}),
                                    AtlasEntityHeader)

    def get_entity_header_by_attribute(self, type_name, uniq_attributes):
        query_params = attributes_to_params(uniq_attributes)

        return self.client.call_api(
            EntityClient.GET_ENTITY_HEADER_BY_UNIQUE_ATTRIBUTE.format_path({'type_name': type_name}),
            AtlasEntityHeader, query_params)

    def get_audit_events(self, guid, start_key, audit_action, count):
        query_params = {"startKey": start_key, "count": count}

        if audit_action is not None:
            query_params["auditAction"] = audit_action

        return self.client.call_api(EntityClient.GET_AUDIT_EVENTS.format_path({'guid': guid}), list, query_params)

    def create_entity(self, entity):
        return self.client.call_api(EntityClient.CREATE_ENTITY, EntityMutationResponse, None, entity)

    def create_entities(self, atlas_entities):
        return self.client.call_api(EntityClient.CREATE_ENTITIES, EntityMutationResponse, None, atlas_entities)

    def update_entity(self, entity):
        return self.client.call_api(EntityClient.UPDATE_ENTITY, EntityMutationResponse, None, entity)

    def update_entities(self, atlas_entities):
        return self.client.call_api(EntityClient.UPDATE_ENTITY, EntityMutationResponse, None, atlas_entities)

    def partial_update_entity_by_guid(self, entity_guid, attr_value, attr_name):
        query_params = {"name": attr_name}

        return self.client.call_api(
            EntityClient.PARTIAL_UPDATE_ENTITY_BY_GUID.format_path({'entity_guid': entity_guid}),
            EntityMutationResponse, query_params, attr_value)

    def delete_entity_by_guid(self, guid):
        return self.client.call_api(EntityClient.DELETE_ENTITY_BY_GUID.format_path_with_params(guid),
                                    EntityMutationResponse)

    def delete_entity_by_attribute(self, type_name, uniq_attributes):
        query_param = attributes_to_params(uniq_attributes)

        return self.client.call_api(EntityClient.DELETE_ENTITY_BY_ATTRIBUTE.format_path_with_params(type_name),
                                    EntityMutationResponse, query_param)

    def delete_entities_by_guids(self, guids):
        query_params = {"guid": guids}

        return self.client.call_api(EntityClient.DELETE_ENTITIES_BY_GUIDS, EntityMutationResponse, query_params)

    def purge_entities_by_guids(self, guids):
        return self.client.call_api(EntityClient.PURGE_ENTITIES_BY_GUIDS, EntityMutationResponse, None, guids)

    # Entity-classification APIs

    def get_classifications(self, guid):
        return self.client.call_api(EntityClient.GET_CLASSIFICATIONS.format_path({'guid': guid}), AtlasClassifications)

    def get_entity_classifications(self, entity_guid, classification_name):
        return self.client.call_api(EntityClient.GET_FROM_CLASSIFICATION.format_path(
            {'entity_guid': entity_guid, 'classification': classification_name}), AtlasClassifications)

    def add_classification(self, request):
        self.client.call_api(EntityClient.ADD_CLASSIFICATION, None, None, request)

    def add_classifications_by_guid(self, guid, classifications):
        self.client.call_api(EntityClient.ADD_CLASSIFICATIONS.format_path({'guid': guid}), None, None, classifications)

    def add_classifications_by_type(self, type_name, uniq_attributes, classifications):
        query_param = attributes_to_params(uniq_attributes)

        self.client.call_api(
            EntityClient.ADD_CLASSIFICATION_BY_TYPE_AND_ATTRIBUTE.format_path({'type_name': type_name}),
            None, query_param, classifications)

    def update_classifications(self, guid, classifications):
        self.client.call_api(EntityClient.UPDATE_CLASSIFICATIONS.format_path(
            {'guid': guid}), None, None, classifications)

    def update_classifications_by_attr(self, type_name, uniq_attributes, classifications):
        query_param = attributes_to_params(uniq_attributes)

        self.client.call_api(
            EntityClient.UPDATE_CLASSIFICATION_BY_TYPE_AND_ATTRIBUTE.format_path({'type_name': type_name}),
            None, query_param, classifications)

    def set_classifications(self, entity_headers):
        return self.client.call_api(EntityClient.UPDATE_BULK_SET_CLASSIFICATIONS, str, None, entity_headers)

    def delete_classification(self, guid, classification_name):
        query = {'guid': guid, 'classification_name': classification_name}

        return self.client.call_api(EntityClient.DELETE_CLASSIFICATION.format_path(query))

    def delete_classifications(self, guid, classifications):
        for atlas_classification in classifications:
            query = {'guid': guid, 'classification_name': atlas_classification.typeName}

            self.client.call_api(EntityClient.DELETE_CLASSIFICATION.format_path(query))

    def remove_classification(self, entity_guid, classification_name, associated_entity_guid):
        query = {'guid': entity_guid, 'classification_name': classification_name}

        self.client.call_api(EntityClient.DELETE_CLASSIFICATION.formart_path(query), None, None, associated_entity_guid)

    def remove_classification_by_name(self, type_name, uniq_attributes, classification_name):
        query_params = attributes_to_params(uniq_attributes)
        query = {'type_name': type_name, 'classification_name': classification_name}

        self.client.call_api(EntityClient.DELETE_CLASSIFICATION_BY_TYPE_AND_ATTRIBUTE.format_path(
            {query}), None, query_params)

    def get_entity_headers(self, tag_update_start_time):
        query_params = {"tagUpdateStartTime": tag_update_start_time}

        return self.client.call_api(EntityClient.GET_BULK_HEADERS, AtlasEntityHeaders, query_params)

    # Business attributes APIs
    def add_or_update_business_attributes(self, entity_guid, is_overwrite, business_attributes):
        query_params = {"isOverwrite": is_overwrite}

        self.client.call_api(EntityClient.ADD_BUSINESS_ATTRIBUTE.format_path({'entity_guid': entity_guid}), None,
                             query_params, business_attributes)

    def add_or_update_business_attributes_bm_name(self, entity_guid, bm_name, business_attributes):
        query = {'entity_guid': entity_guid, 'bm_name': bm_name}

        self.client.call_api(EntityClient.ADD_BUSINESS_ATTRIBUTE_BY_NAME.format_path(query),
                             None, None, business_attributes)

    def remove_business_attributes(self, entity_guid, business_attributes):
        self.client.call_api(EntityClient.DELETE_BUSINESS_ATTRIBUTE.format_path({'entity_guid': entity_guid}), None,
                             None, business_attributes)

    def remove_business_attributes_bm_name(self, entity_guid, bm_name, business_attributes):
        query = {'entity_guid': entity_guid, 'bm_name': bm_name}

        self.client.call_api(EntityClient.DELETE_BUSINESS_ATTRIBUTE_BY_NAME.format_path(query),
                             None, None, business_attributes)

    # Labels APIs
    def add_labels_by_guid(self, entity_guid, labels):
        self.client.call_api(EntityClient.ADD_LABELS.format_path({'entity_guid': entity_guid}), None, None, labels)

    def add_labels_by_name(self, type_name, uniq_attributes, labels):
        query_param = attributes_to_params(uniq_attributes)

        self.client.call_api(EntityClient.SET_LABELS_BY_UNIQUE_ATTRIBUTE.format_path({'type_name': type_name}), None,
                             query_param, labels)

    def remove_labels_by_guid(self, entity_guid, labels):
        self.client.call_api(EntityClient.DELETE_LABELS.format_path({'entity_guid': entity_guid}), None, None, labels)

    def remove_labels_by_name(self, type_name, uniq_attributes, labels):
        query_param = attributes_to_params(uniq_attributes)

        self.client.call_api(EntityClient.DELETE_LABELS_BY_UNIQUE_ATTRIBUTE.format({'type_name': type_name}), None,
                             query_param, labels)

    def set_labels_by_guid(self, entity_guid, labels):
        self.client.call_api(EntityClient.SET_LABELS.format({'entity_guid': entity_guid}), None, None, labels)

    def set_labels_by_name(self, type_name, uniq_attributes, labels):
        query_param = attributes_to_params(uniq_attributes)

        self.client.call_api(EntityClient.ADD_LABELS_BY_UNIQUE_ATTRIBUTE.format(
            {'type_name': type_name}), None, query_param, labels)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.admin import AtlasAdminMetrics
from apache_atlas.utils import API, BASE_URI, HTTPStatus, HTTPMethod


class AdminClient:
    BASE_ADMIN_URL = BASE_URI + "admin/"

    GET_METRICS_API = API(BASE_ADMIN_URL + "metrics", HTTPMethod.GET, HTTPStatus.OK)

    def __init__(self, client):
        self.client = client

    def get_metrics(self):
        return self.client.call_api(AdminClient.GET_METRICS_API, AtlasAdminMetrics)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicabwle law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import enum

from apache_atlas.model.enums import QueryType
from apache_atlas.model.instance import AtlasEntityHeader
from apache_atlas.model.misc import AtlasBase
from apache_atlas.model.misc import AtlasBaseModelObject
from apache_atlas.utils import non_null
from apache_atlas.utils import type_coerce
from apache_atlas.utils import type_coerce_dict
from apache_atlas.utils import type_coerce_dict_list
from apache_atlas.utils import type_coerce_list


class AtlasAggregationEntry(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.name = attrs.get('name')
        self.name = attrs.get('count')


class AtlasQuickSearchResult(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.searchResults = attrs.get('searchResults')
        self.aggregationMetrics = attrs.get('aggregationMetrics')

    def type_coerce_attrs(self):
        super(AtlasQuickSearchResult, self).type_coerce_attrs()

        self.searchResults = type_coerce(self.searchResults, AtlasSearchResult)
        self.aggregationMetrics = type_coerce_dict_list(self.aggregationMetrics, AtlasAggregationEntry)


class AtlasSearchResult(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.queryType = non_null(attrs.get('queryType'), QueryType.BASIC.name)
        self.searchParameters = attrs.get('searchParameters')
        self.queryText = attrs.get('queryText')
        self.type = attrs.get('type')
        self.classification = attrs.get('classification')
        self.entities = attrs.get('entities')
        self.attributes = attrs.get('attributes')
        self.fullTextResult = attrs.get('fullTextResult')
        self.referredEntities = attrs.get('referredEntities')
        self.approximateCount = non_null(attrs.get('approximateCount'), -1)

    def type_coerce_attrs(self):
        super(AtlasSearchResult, self).type_coerce_attrs()

        self.entities = type_coerce_list(self.entities, AtlasEntityHeader)
        self.attributes = type_coerce(self.attributes, AttributeSearchResult)
        self.referredEntities = type_coerce_dict(self.referredEntities, AtlasEntityHeader)


class AttributeSearchResult(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.name = attrs.get('name')
        self.values = attrs.get('values')


class AtlasFullTextResult(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.entity = attrs.get('entity')
        self.score = attrs.get('score')

    def type_coerce_attrs(self):
        super(AtlasFullTextResult, self).type_coerce_attrs()

        self.entity = type_coerce(self.criterion, AtlasEntityHeader)


class AtlasSuggestionsResult(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.suggestions = attrs.get('suggestions')
        self.prefixString = attrs.get('prefixString')
        self.fieldName = attrs.get('fieldName')


class QuickSearchParameters(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.query = attrs.get('query')
        self.typeName = attrs.get('typeName')
        self.entityFilters = attrs.get('entityFilters')
        self.includeSubTypes = attrs.get('includeSubTypes')
        self.excludeDeletedEntities = attrs.get('excludeDeletedEntities')
        self.offset = attrs.get('offset')
        self.limit = attrs.get('limit')
        self.attributes = attrs.get('attributes')

    def type_coerce_attrs(self):
        super(QuickSearchParameters, self).type_coerce_attrs()

        self.entityFilters = type_coerce(self.entityFilters, FilterCriteria)


class SearchParameters(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.query = attrs.get('query')
        self.typeName = attrs.get('typeName')
        self.classification = attrs.get('classification')
        self.termName = attrs.get('termName')
        self.sortBy = attrs.get('sortBy')
        self.excludeDeletedEntities = attrs.get('excludeDeletedEntities')
        self.includeClassificationAttributes = attrs.get('includeClassificationAttributes')
        self.includeSubTypes = non_null(attrs.get('includeSubTypes'), True)
        self.includeSubClassifications = non_null(attrs.get('includeSubClassifications'), True)
        self.limit = attrs.get('limit')
        self.offset = attrs.get('offset')
        self.entityFilters = attrs.get('entityFilters')
        self.tagFilters = attrs.get('tagFilters')
        self.attributes = attrs.get('attributes')
        self.sortOrder = attrs.get('sortOrder')

    def type_coerce_attrs(self):
        super(SearchParameters, self).type_coerce_attrs()

        self.entityFilters = type_coerce(self.entityFilters, FilterCriteria)
        self.tagFilters = type_coerce(self.tagFilters, FilterCriteria)


class FilterCriteria(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.attributeName = attrs.get('attributeName')
        self.operator = attrs.get('operator')
        self.attributeValue = attrs.get('attributeValue')
        self.condition = attrs.get('condition')
        self.criterion = attrs.get('criterion')

    def type_coerce_attrs(self):
        super(FilterCriteria, self).type_coerce_attrs()

        self.criterion = type_coerce(self.criterion, FilterCriteria)


class AtlasUserSavedSearch(AtlasBaseModelObject):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBaseModelObject.__init__(self, attrs)

        self.ownerName = attrs.get('ownerName')
        self.name = attrs.get('name')
        self.searchType = attrs.get('searchType')
        self.searchParameters = attrs.get('searchParameters')
        self.uiParameters = attrs.get('uiParameters')

    def type_coerce_attrs(self):
        super(AtlasUserSavedSearch, self).type_coerce_attrs()

        self.searchParameters = type_coerce(self.searchParameters, SearchParameters)


class Operator(enum.Enum):
    LT = ("<", "lt")
    GT = ('>', 'gt')
    LTE = ('<=', 'lte')
    GTE = ('>=', 'gte')
    EQ = ('=', 'eq')
    NEQ = ('!=', 'neq')
    IN = ('in', 'IN')
    LIKE = ('like', 'LIKE')
    STARTS_WITH = ('startsWith', 'STARTSWITH', 'begins_with', 'BEGINS_WITH')
    ENDS_WITH = ('endsWith', 'ENDSWITH', 'ends_with', 'ENDS_WITH')
    CONTAINS = ('contains', 'CONTAINS')
    NOT_CONTAINS = ('not_contains', 'NOT_CONTAINS')
    CONTAINS_ANY = ('containsAny', 'CONTAINSANY', 'contains_any', 'CONTAINS_ANY')
    CONTAINS_ALL = ('containsAll', 'CONTAINSALL', 'contains_all', 'CONTAINS_ALL')
    IS_NULL = ('isNull', 'ISNULL', 'is_null', 'IS_NULL')
    NOT_NULL = ('notNull', 'NOTNULL', 'not_null', 'NOT_NULL')


class SortOrder(enum.Enum):
    sort_order = enum.Enum('sort_order', 'ASCENDING DESCENDING', module=__name__)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import copy
import json
import logging

from requests import Session
from urllib.parse import urljoin

from apache_atlas.client.admin import AdminClient
from apache_atlas.client.discovery import DiscoveryClient
from apache_atlas.client.entity import EntityClient
from apache_atlas.client.glossary import GlossaryClient
from apache_atlas.client.lineage import LineageClient
from apache_atlas.client.relationship import RelationshipClient
from apache_atlas.client.typedef import TypeDefClient
from apache_atlas.exceptions import AtlasServiceException
from apache_atlas.utils import HTTPMethod
from apache_atlas.utils import HTTPStatus
from apache_atlas.utils import type_coerce

log = logging.getLogger('apache_atlas')


class AtlasClient:
    def __init__(self, host, auth):
        session = Session()
        session.auth = auth

        self.host = host.rstrip('/')
        self.session = session
        self.request_params = {'headers': {}}
        self.typedef = TypeDefClient(self)
        self.entity = EntityClient(self)
        self.lineage = LineageClient(self)
        self.discovery = DiscoveryClient(self)
        self.glossary = GlossaryClient(self)
        self.relationship = RelationshipClient(self)
        self.admin = AdminClient(self)

        logging.getLogger("requests").setLevel(logging.WARNING)

    def call_api(self, api, response_type=None, query_params=None, request_obj=None):
        params = copy.deepcopy(self.request_params)
        path = urljoin(self.host, api.path.lstrip('/'))

        params['headers']['Accept'] = api.consumes
        params['headers']['Content-type'] = api.produces

        if query_params is not None:
            params['params'] = query_params

        if request_obj is not None:
            params['data'] = json.dumps(request_obj)

        if log.isEnabledFor(logging.DEBUG):
            log.debug("------------------------------------------------------")
            log.debug("Call         : %s %s", api.method, path)
            log.debug("Content-type : %s", api.consumes)
            log.debug("Accept       : %s", api.produces)

        response = None

        if api.method == HTTPMethod.GET:
            response = self.session.get(path, **params)
        elif api.method == HTTPMethod.POST:
            response = self.session.post(path, **params)
        elif api.method == HTTPMethod.PUT:
            response = self.session.put(path, **params)
        elif api.method == HTTPMethod.DELETE:
            response = self.session.delete(path, **params)

        if response is not None:
            log.debug("HTTP Status: %s", response.status_code)

        if response is None:
            return None
        elif response.status_code == api.expected_status:
            if response_type is None:
                return None

            try:
                if response.content is not None:
                    if log.isEnabledFor(logging.DEBUG):
                        log.debug("<== __call_api(%s,%s,%s), result = %s", vars(api), params, request_obj, response)

                        log.debug(response.json())
                    if response_type == str:
                        return json.dumps(response.json())

                    return type_coerce(response.json(), response_type)
                else:
                    return None
            except Exception as e:
                log.exception("Exception occurred while parsing response with msg: %s", e)

                raise AtlasServiceException(api, response)
        elif response.status_code == HTTPStatus.SERVICE_UNAVAILABLE:
            log.error("Atlas Service unavailable. HTTP Status: %s", HTTPStatus.SERVICE_UNAVAILABLE)

            return None
        else:
            raise AtlasServiceException(api, response)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.relationship import AtlasRelationship
from apache_atlas.model.relationship import AtlasRelationshipWithExtInfo
from apache_atlas.utils import API
from apache_atlas.utils import BASE_URI
from apache_atlas.utils import HTTPMethod
from apache_atlas.utils import HTTPStatus


class RelationshipClient:
    RELATIONSHIPS_URI = BASE_URI + "v2/relationship/"
    BULK_HEADERS = "bulk/headers"
    BULK_SET_CLASSIFICATIONS = "bulk/setClassifications"

    GET_RELATIONSHIP_BY_GUID = API(RELATIONSHIPS_URI + "guid", HTTPMethod.GET, HTTPStatus.OK)
    CREATE_RELATIONSHIP = API(RELATIONSHIPS_URI, HTTPMethod.POST, HTTPStatus.OK)
    UPDATE_RELATIONSHIP = API(RELATIONSHIPS_URI, HTTPMethod.PUT, HTTPStatus.OK)
    DELETE_RELATIONSHIP_BY_GUID = API(RELATIONSHIPS_URI + "guid", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)

    def __init__(self, client):
        self.client = client

    def get_relationship_by_guid(self, guid):
        return self.client.call_api(RelationshipClient.GET_RELATIONSHIP_BY_GUID.format_path_with_params(guid),
                                    AtlasRelationshipWithExtInfo)

    def get_relationship_by_guid_with_ext_info(self, guid, extended_info):
        query_params = {"extendedInfo": extended_info}

        return self.client.call_api(RelationshipClient.GET_RELATIONSHIP_BY_GUID.format_path_with_params(guid),
                                    AtlasRelationshipWithExtInfo, query_params)

    def create_relationship(self, relationship):
        return self.client.call_api(RelationshipClient.CREATE_RELATIONSHIP, AtlasRelationship, None, relationship)

    def update_relationship(self, relationship):
        return self.client.call_api(RelationshipClient.UPDATE_RELATIONSHIP, AtlasRelationship, relationship)

    def delete_relationship_by_guid(self, guid):
        return self.client.call_api(RelationshipClient.DELETE_RELATIONSHIP_BY_GUID.format_path_with_params(guid))



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicabwle law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.discovery import SearchParameters
from apache_atlas.model.misc import AtlasBaseModelObject
from apache_atlas.utils import type_coerce


class AtlasUserSavedSearch(AtlasBaseModelObject):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBaseModelObject.__init__(self, attrs)

        self.ownerName = attrs.get('ownerName')
        self.name = attrs.get('name')
        self.searchType = attrs.get('searchType')
        self.searchParameters = attrs.get('searchParameters')
        self.uiParameters = attrs.get('uiParameters')

    def type_coerce_attrs(self):
        super(AtlasUserSavedSearch, self).type_coerce_attrs()

        self.searchParameters = type_coerce(self.searchParameters, SearchParameters)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.instance import AtlasBase
from apache_atlas.model.instance import AtlasEntityHeader
from apache_atlas.utils import type_coerce_dict
from apache_atlas.utils import type_coerce_list


class AtlasLineageInfo(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.baseEntityGuid = attrs.get('baseEntityGuid')
        self.lineageDirection = attrs.get('lineageDirection')
        self.lineageDepth = attrs.get('lineageDepth')
        self.guidEntityMap = attrs.get('guidEntityMap')
        self.relations = attrs.get('relations')

    def type_coerce_attrs(self):
        super(AtlasLineageInfo, self).type_coerce_attrs()

        self.guidEntityMap = type_coerce_dict(self.guidEntityMap, AtlasEntityHeader)
        self.relations = type_coerce_list(self.relations, LineageRelation)


class LineageRelation(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.fromEntityId = attrs.get('fromEntityId')
        self.toEntityId = attrs.get('toEntityId')
        self.relationshipId = attrs.get('relationshipId')



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.misc import AtlasBase


class AtlasAdminMetrics(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        _data = attrs.get('data', {})

        self.general = _data.get('general', {})
        self.tag = _data.get('tag', {})
        self.entity = _data.get('entity', {})
        self.system = _data.get('system', {})



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicabwle law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import enum


class AtlasTermAssignmentStatus(enum.Enum):
    DISCOVERED = 0
    PROPOSED = 1
    IMPORTED = 2
    VALIDATED = 3
    DEPRECATED = 4
    OBSOLETE = 5
    OTHER = 6


class AtlasTermRelationshipStatus(enum.Enum):
    DRAFT = 0
    ACTIVE = 1
    DEPRECATED = 2
    OBSOLETE = 3
    OTHER = 99


class TypeCategory(enum.Enum):
    PRIMITIVE = 0
    OBJECT_ID_TYPE = 1
    ENUM = 2
    STRUCT = 3
    CLASSIFICATION = 4
    ENTITY = 5
    ARRAY = 6
    MAP = 7
    RELATIONSHIP = 8
    BUSINESS_METADATA = 9


class Cardinality(enum.Enum):
    SINGLE = 0
    LIST = 1
    SET = 2


class Condition(enum.Enum):
    AND = 0
    OR = 1


class EntityOperation(enum.Enum):
    CREATE = 0
    UPDATE = 1
    PARTIAL_UPDATE = 2
    DELETE = 3
    PURGE = 4


class EntityStatus(enum.Enum):
    ACTIVE = 0
    DELETED = 1
    PURGED = 2


class IndexType(enum.Enum):
    DEFAULT = 0
    STRING = 1


class LineageDirection(enum.Enum):
    INPUT = 0
    OUTPUT = 1
    BOTH = 2


class Operator(enum.Enum):
    LT = ("<", "lt")
    GT = ('>', 'gt')
    LTE = ('<=', 'lte')
    GTE = ('>=', 'gte')
    EQ = ('=', 'eq')
    NEQ = ('!=', 'neq')
    IN = ('in', 'IN')
    LIKE = ('like', 'LIKE')
    STARTS_WITH = ('startsWith', 'STARTSWITH', 'begins_with', 'BEGINS_WITH')
    ENDS_WITH = ('endsWith', 'ENDSWITH', 'ends_with', 'ENDS_WITH')
    CONTAINS = ('contains', 'CONTAINS')
    NOT_CONTAINS = ('not_contains', 'NOT_CONTAINS')
    CONTAINS_ANY = ('containsAny', 'CONTAINSANY', 'contains_any', 'CONTAINS_ANY')
    CONTAINS_ALL = ('containsAll', 'CONTAINSALL', 'contains_all', 'CONTAINS_ALL')
    IS_NULL = ('isNull', 'ISNULL', 'is_null', 'IS_NULL')
    NOT_NULL = ('notNull', 'NOTNULL', 'not_null', 'NOT_NULL')


class PropagateTags(enum.Enum):
    NONE = 0
    ONE_TO_TWO = 1
    TWO_TO_ONE = 2
    BOTH = 3


class QueryType(enum.Enum):
    DSL = 0
    FULL_TEXT = 1
    GREMLIN = 2
    BASIC = 3
    ATTRIBUTE = 4
    RELATIONSHIP = 5


class RelationshipCategory(enum.Enum):
    ASSOCIATION = 0
    AGGREGATION = 1
    COMPOSITION = 2


class RelationshipStatus(enum.Enum):
    ACTIVE = 0
    DELETED = 1


class SavedSearchType(enum.Enum):
    BASIC = 0
    ADVANCED = 1


class SortOrder(enum.Enum):
    ASCENDING = 0
    DESCENDING = 1


class SortType(enum.Enum):
    NONE = 0
    ASC = 1
    DESC = 2



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import enum
import time
from functools import reduce

BASE_URI = "api/atlas/"
APPLICATION_JSON = 'application/json'
APPLICATION_OCTET_STREAM = 'application/octet-stream'
MULTIPART_FORM_DATA = 'multipart/form-data'
PREFIX_ATTR = "attr:"
PREFIX_ATTR_ = "attr_"

s_nextId = milliseconds = int(round(time.time() * 1000)) + 1


def next_id():
    global s_nextId

    s_nextId += 1

    return "-" + str(s_nextId)


def list_attributes_to_params(attributes_list, query_params=None):
    if query_params is None:
        query_params = {}

    for i, attr in enumerate(attributes_list):
        for key, value in attr.items():
            new_key = PREFIX_ATTR_ + str(i) + ":" + key
            query_params[new_key] = value

    return query_params


def attributes_to_params(attributes, query_params=None):
    if query_params is None:
        query_params = {}

    if attributes:
        for key, value in attributes:
            new_key = PREFIX_ATTR + key
            query_params[new_key] = value

    return query_params


def non_null(obj, defValue):
    return obj if obj is not None else defValue


def type_coerce(obj, objType):
    if isinstance(obj, objType):
        ret = obj
    elif isinstance(obj, dict):
        ret = objType(obj)

        ret.type_coerce_attrs()
    else:
        ret = None

    return ret


def type_coerce_list(obj, objType):
    if isinstance(obj, list):
        ret = []
        for entry in obj:
            ret.append(type_coerce(entry, objType))
    else:
        ret = None

    return ret


def type_coerce_dict(obj, objType):
    if isinstance(obj, dict):
        ret = {}
        for k, v in obj.items():
            ret[k] = type_coerce(v, objType)
    else:
        ret = None

    return ret


def type_coerce_dict_list(obj, objType):
    if isinstance(obj, dict):
        ret = {}
        for k, v in obj.items():
            ret[k] = type_coerce_list(v, objType)
    else:
        ret = None

    return ret


class API:
    def __init__(self, path, method, expected_status, consumes=APPLICATION_JSON, produces=APPLICATION_JSON):
        self.path = path
        self.method = method
        self.expected_status = expected_status
        self.consumes = consumes
        self.produces = produces

    def multipart_urljoin(self, base_path, *path_elems):
        """Join a base path and multiple context path elements. Handle single
        leading and trailing `/` characters transparently.

        Args:
            base_path (string): the base path or url (ie. `http://atlas/v2/`)
            *path_elems (string): multiple relative path elements (ie. `/my/relative`, `/path`)

        Returns:
            string: the result of joining the base_path with the additional path elements
        """
        def urljoin_pair(left, right):
            return "/".join([left.rstrip('/'), right.strip('/')])

        return reduce(urljoin_pair, path_elems, base_path)

    def format_path(self, params):
        return API(self.path.format(**params), self.method, self.expected_status, self.consumes, self.produces)

    def format_path_with_params(self, *params):
        request_path = self.multipart_urljoin(self.path, *params)
        return API(request_path, self.method, self.expected_status, self.consumes, self.produces)


class HTTPMethod(enum.Enum):
    GET = "GET"
    PUT = "PUT"
    POST = "POST"
    DELETE = "DELETE"


class HTTPStatus:
    OK = 200
    NO_CONTENT = 204
    SERVICE_UNAVAILABLE = 503



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class AtlasServiceException(Exception):
    """Exception raised for errors in API calls.

    Attributes:
        api -- api endpoint which caused the error
        response -- response from the server
    """

    def __init__(self, api, response):
        body = ""

        if response.content:
            try:
                body = response.json()
            except Exception:
                body = response.content

        msg = "Metadata service API {method} {path} failed with status {status}. Response body is: {body}".format(**{'method': api.method, 'path': api.path, 'status': response.status_code, 'body': body})

        Exception.__init__(self, msg)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.misc import AtlasBase, AtlasBaseModelObject
from apache_atlas.utils import type_coerce, type_coerce_dict, type_coerce_list


class AtlasGlossaryBaseObject(AtlasBaseModelObject):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBaseModelObject.__init__(self, attrs)

        self.qualifiedName = attrs.get('qualifiedName')
        self.name = attrs.get('name')
        self.shortDescription = attrs.get('shortDescription')
        self.longDescription = attrs.get('longDescription')
        self.additionalAttributes = attrs.get('additionalAttributes')
        self.classifications = attrs.get('classifications')

    def type_coerce_attrs(self):
        # This is to avoid the circular dependencies that instance.py and glossary.py has.
        import apache_atlas.model.instance as instance

        super(AtlasGlossaryBaseObject, self).type_coerce_attrs()
        self.classifications = type_coerce_list(self.classifications, instance.AtlasClassification)


class AtlasGlossary(AtlasGlossaryBaseObject):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasGlossaryBaseObject.__init__(self, attrs)

        self.language = attrs.get('language')
        self.usage = attrs.get('usage')
        self.terms = attrs.get('terms')
        self.categories = attrs.get('categories')

    def type_coerce_attrs(self):
        super(AtlasGlossary, self).type_coerce_attrs()

        self.terms = type_coerce_list(self.classifications, AtlasRelatedTermHeader)
        self.categories = type_coerce_list(self.categories, AtlasRelatedCategoryHeader)


class AtlasGlossaryExtInfo(AtlasGlossary):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasGlossary.__init__(self, attrs)

        self.termInfo = attrs.get('termInfo')
        self.categoryInfo = attrs.get('categoryInfo')

    def type_coerce_attrs(self):
        super(AtlasGlossaryExtInfo, self).type_coerce_attrs()

        self.termInfo = type_coerce_dict(self.termInfo, AtlasGlossaryTerm)
        self.categoryInfo = type_coerce_dict(self.categoryInfo, AtlasGlossaryCategory)


class AtlasGlossaryCategory(AtlasGlossaryBaseObject):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasGlossaryBaseObject.__init__(self, attrs)

        # Inherited attributes from relations
        self.anchor = attrs.get('anchor')

        # Category hierarchy links
        self.parentCategory = attrs.get('parentCategory')
        self.childrenCategories = attrs.get('childrenCategories')

        # Terms associated with this category
        self.terms = attrs.get('terms')

    def type_coerce_attrs(self):
        super(AtlasGlossaryCategory, self).type_coerce_attrs()

        self.anchor = type_coerce(self.anchor, AtlasGlossaryHeader)
        self.parentCategory = type_coerce(self.parentCategory, AtlasRelatedCategoryHeader)
        self.childrenCategories = type_coerce_list(self.childrenCategories, AtlasRelatedCategoryHeader)
        self.terms = type_coerce_list(self.terms, AtlasRelatedTermHeader)


class AtlasGlossaryTerm(AtlasGlossaryBaseObject):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasGlossaryBaseObject.__init__(self, attrs)

        # Core attributes
        self.examples = attrs.get('examples')
        self.abbreviation = attrs.get('abbreviation')
        self.usage = attrs.get('usage')

        # Attributes derived from relationships
        self.anchor = attrs.get('anchor')
        self.assignedEntities = attrs.get('assignedEntities')
        self.categories = attrs.get('categories')

        # Related Terms
        self.seeAlso = attrs.get('seeAlso')

        # Term Synonyms
        self.synonyms = attrs.get('synonyms')

        # Term antonyms
        self.antonyms = attrs.get('antonyms')

        # Term preference
        self.preferredTerms = attrs.get('preferredTerms')
        self.preferredToTerms = attrs.get('preferredToTerms')

        # Term replacements
        self.replacementTerms = attrs.get('replacementTerms')
        self.replacedBy = attrs.get('replacedBy')

        # Term translations
        self.translationTerms = attrs.get('translationTerms')
        self.translatedTerms = attrs.get('translatedTerms')

        # Term classification
        self.isA = attrs.get('isA')
        self.classifies = attrs.get('classifies')

        # Values for terms
        self.validValues = attrs.get('validValues')
        self.validValuesFor = attrs.get('validValuesFor')

    def type_coerce_attrs(self):
        super(AtlasGlossaryTerm, self).type_coerce_attrs()

        # This is to avoid the circular dependencies that instance.py and glossary.py has.
        import apache_atlas.model.instance as instance

        self.anchor = type_coerce(self.anchor, AtlasGlossaryHeader)
        self.assignedEntities = type_coerce_list(self.assignedEntities, instance.AtlasRelatedObjectId)
        self.categories = type_coerce_list(self.categories, AtlasTermCategorizationHeader)
        self.seeAlso = type_coerce_list(self.seeAlso, AtlasRelatedTermHeader)
        self.synonyms = type_coerce_list(self.synonyms, AtlasRelatedTermHeader)
        self.antonyms = type_coerce_list(self.antonyms, AtlasRelatedTermHeader)
        self.preferredTerms = type_coerce_list(self.preferredTerms, AtlasRelatedTermHeader)
        self.preferredToTerms = type_coerce_list(self.preferredToTerms, AtlasRelatedTermHeader)
        self.replacementTerms = type_coerce_list(self.replacementTerms, AtlasRelatedTermHeader)
        self.replacedBy = type_coerce_list(self.replacedBy, AtlasRelatedTermHeader)
        self.translationTerms = type_coerce_list(self.translationTerms, AtlasRelatedTermHeader)
        self.isA = type_coerce_list(self.isA, AtlasRelatedTermHeader)
        self.classifies = type_coerce_list(self.classifies, AtlasRelatedTermHeader)
        self.validValues = type_coerce_list(self.validValues, AtlasRelatedTermHeader)
        self.validValuesFor = type_coerce_list(self.validValuesFor, AtlasRelatedTermHeader)


class AtlasGlossaryHeader(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.glossaryGuid = attrs.get('glossaryGuid')
        self.relationGuid = attrs.get('relationGuid')
        self.displayText = attrs.get('displayText')


class AtlasRelatedCategoryHeader(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.categoryGuid = attrs.get('categoryGuid')
        self.parentCategoryGuid = attrs.get('parentCategoryGuid')
        self.relationGuid = attrs.get('relationGuid')
        self.displayText = attrs.get('displayText')
        self.description = attrs.get('description')


class AtlasRelatedTermHeader(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.termGuid = attrs.get('termGuid')
        self.relationGuid = attrs.get('relationGuid')
        self.displayText = attrs.get('displayText')
        self.description = attrs.get('description')
        self.expression = attrs.get('expression')
        self.steward = attrs.get('steward')
        self.source = attrs.get('source')
        self.status = attrs.get('status')


class AtlasTermAssignmentHeader(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.termGuid = attrs.get('termGuid')
        self.relationGuid = attrs.get('relationGuid')
        self.description = attrs.get('description')
        self.displayText = attrs.get('displayText')
        self.expression = attrs.get('expression')
        self.createdBy = attrs.get('createdBy')
        self.steward = attrs.get('steward')
        self.source = attrs.get('source')
        self.confidence = attrs.get('confidence')


class AtlasTermCategorizationHeader(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.categoryGuid = attrs.get('categoryGuid')
        self.relationGuid = attrs.get('relationGuid')
        self.description = attrs.get('description')
        self.displayText = attrs.get('displayText')
        self.status = attrs.get('status')



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging

from apache_atlas.model.enums import TypeCategory
from apache_atlas.model.misc import AtlasBase
from apache_atlas.utils import non_null
from apache_atlas.utils import type_coerce
from apache_atlas.utils import type_coerce_dict_list
from apache_atlas.utils import type_coerce_list

LOG = logging.getLogger('apache_atlas')


class AtlasBaseTypeDef(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.category = attrs.get('category')
        self.guid = attrs.get('guid')
        self.createdBy = attrs.get('createdBy')
        self.updatedBy = attrs.get('updatedBy')
        self.createTime = attrs.get('createTime')
        self.updateTime = attrs.get('updateTime')
        self.version = attrs.get('version')
        self.name = attrs.get('name')
        self.description = attrs.get('description')
        self.typeVersion = attrs.get('typeVersion')
        self.serviceType = attrs.get('serviceType')
        self.options = attrs.get('options')


class AtlasEnumDef(AtlasBaseTypeDef):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBaseTypeDef.__init__(self, attrs)

        self.elementDefs = attrs.get('elementDefs')
        self.defaultValue = attrs.get('defaultValue')

    def type_coerce_attrs(self):
        super(AtlasEnumDef, self).type_coerce_attrs()

        self.elementDefs = type_coerce_list(self.elementDefs, AtlasEnumElementDef)


class AtlasStructDef(AtlasBaseTypeDef):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBaseTypeDef.__init__(self, attrs)

        self.category = non_null(attrs.get('category'), TypeCategory.STRUCT.name)
        self.attributeDefs = attrs.get('attributeDefs')

    def type_coerce_attrs(self):
        super(AtlasStructDef, self).type_coerce_attrs()

        self.attributeDefs = type_coerce_list(self.attributeDefs, AtlasAttributeDef)


class AtlasClassificationDef(AtlasStructDef):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStructDef.__init__(self, attrs)

        self.category = TypeCategory.CLASSIFICATION.name
        self.superTypes = attrs.get('superTypes')
        self.entityTypes = attrs.get('entityTypes')
        self.subTypes = attrs.get('subTypes')


class AtlasEntityDef(AtlasStructDef):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStructDef.__init__(self, attrs)

        self.category = TypeCategory.ENTITY.name
        self.superTypes = attrs.get('superTypes')
        self.subTypes = attrs.get('subTypes')
        self.relationshipAttributeDefs = attrs.get('relationshipAttributeDefs')
        self.businessAttributeDefs = attrs.get('businessAttributeDefs')

    def type_coerce_attrs(self):
        super(AtlasEntityDef, self).type_coerce_attrs()

        self.relationshipAttributeDefs = type_coerce_list(self.relationshipAttributeDefs, AtlasRelationshipAttributeDef)
        self.businessAttributeDefs = type_coerce_dict_list(self.businessAttributeDefs, AtlasAttributeDef)


class AtlasRelationshipDef(AtlasStructDef):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStructDef.__init__(self, attrs)

        self.category = TypeCategory.RELATIONSHIP.name
        self.relationshipCategory = attrs.get('relationshipCategory')
        self.relationshipLabel = attrs.get('relationshipLabel')
        self.propagateTags = attrs.get('propagateTags')
        self.endDef1 = attrs.get('endDef1')
        self.endDef2 = attrs.get('endDef2')

    def type_coerce_attrs(self):
        super(AtlasRelationshipDef, self).type_coerce_attrs()

        self.endDef1 = type_coerce(self.endDef1, AtlasRelationshipEndDef)
        self.endDef2 = type_coerce(self.endDef2, AtlasRelationshipEndDef)


class AtlasBusinessMetadataDef(AtlasStructDef):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStructDef.__init__(self, attrs)

        self.category = TypeCategory.BUSINESS_METADATA.name


class AtlasAttributeDef(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.name = attrs.get('name')
        self.typeName = attrs.get('typeName')
        self.isOptional = attrs.get('isOptional')
        self.cardinality = attrs.get('cardinality')
        self.valuesMinCount = attrs.get('valuesMinCount')
        self.valuesMaxCount = attrs.get('valuesMaxCount')
        self.isUnique = attrs.get('isUnique')
        self.isIndexable = attrs.get('isIndexable')
        self.includeInNotification = attrs.get('includeInNotification')
        self.defaultValue = attrs.get('defaultValue')
        self.description = attrs.get('description')
        self.searchWeight = non_null(attrs.get('searchWeight'), -1)
        self.indexType = attrs.get('indexType')
        self.constraints = attrs.get('constraints')
        self.options = attrs.get('options')
        self.displayName = attrs.get('displayName')

    def type_coerce_attrs(self):
        super(AtlasAttributeDef, self).type_coerce_attrs()

        self.constraints = type_coerce_list(self.constraints, AtlasConstraintDef)


class AtlasConstraintDef(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.type = attrs.get('type')
        self.params = attrs.get('params')


class AtlasEnumElementDef(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.value = attrs.get('value')
        self.description = attrs.get('description')
        self.ordinal = attrs.get('ordinal')


class AtlasRelationshipAttributeDef(AtlasAttributeDef):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasAttributeDef.__init__(self, attrs)

        self.relationshipTypeName = attrs.get('relationshipTypeName')
        self.isLegacyAttribute = attrs.get('isLegacyAttribute')


class AtlasRelationshipEndDef(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.type = attrs.get('type')
        self.name = attrs.get('name')
        self.isContainer = attrs.get('isContainer')
        self.cardinality = attrs.get('cardinality')
        self.isLegacyAttribute = attrs.get('isLegacyAttribute')
        self.description = attrs.get('description')


class AtlasTypesDef(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.enumDefs = attrs.get('enumDefs')
        self.structDefs = attrs.get('structDefs')
        self.classificationDefs = attrs.get('classificationDefs')
        self.entityDefs = attrs.get('entityDefs')
        self.relationshipDefs = attrs.get('relationshipDefs')
        self.businessMetadataDefs = attrs.get('businessMetadataDefs')

    def type_coerce_attrs(self):
        super(AtlasTypesDef, self).type_coerce_attrs()

        self.enumDefs = type_coerce_list(self.enumDefs, AtlasEnumDef)
        self.structDefs = type_coerce_list(self.structDefs, AtlasStructDef)
        self.classificationDefs = type_coerce_list(self.classificationDefs, AtlasClassificationDef)
        self.entityDefs = type_coerce_list(self.entityDefs, AtlasEntityDef)
        self.relationshipDefs = type_coerce_list(self.relationshipDefs, AtlasRelationshipDef)
        self.businessMetadataDefs = type_coerce_list(self.businessMetadataDefs, AtlasBusinessMetadataDef)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.typedef import AtlasBusinessMetadataDef
from apache_atlas.model.typedef import AtlasClassificationDef
from apache_atlas.model.typedef import AtlasEntityDef
from apache_atlas.model.typedef import AtlasEnumDef
from apache_atlas.model.typedef import AtlasRelationshipDef
from apache_atlas.model.typedef import AtlasStructDef
from apache_atlas.model.typedef import AtlasTypesDef
from apache_atlas.utils import API
from apache_atlas.utils import BASE_URI
from apache_atlas.utils import HTTPMethod
from apache_atlas.utils import HTTPStatus


class TypeDefClient:
    TYPES_API = BASE_URI + "v2/types/"
    TYPEDEFS_API = TYPES_API + "typedefs/"
    TYPEDEF_BY_NAME = TYPES_API + "typedef/name"
    TYPEDEF_BY_GUID = TYPES_API + "typedef/guid"
    GET_BY_NAME_TEMPLATE = TYPES_API + "{path_type}/name/{name}"
    GET_BY_GUID_TEMPLATE = TYPES_API + "{path_type}/guid/{guid}"

    GET_TYPEDEF_BY_NAME = API(TYPEDEF_BY_NAME, HTTPMethod.GET, HTTPStatus.OK)
    GET_TYPEDEF_BY_GUID = API(TYPEDEF_BY_GUID, HTTPMethod.GET, HTTPStatus.OK)
    GET_ALL_TYPE_DEFS = API(TYPEDEFS_API, HTTPMethod.GET, HTTPStatus.OK)
    GET_ALL_TYPE_DEF_HEADERS = API(TYPEDEFS_API + "headers", HTTPMethod.GET, HTTPStatus.OK)
    UPDATE_TYPE_DEFS = API(TYPEDEFS_API, HTTPMethod.PUT, HTTPStatus.OK)
    CREATE_TYPE_DEFS = API(TYPEDEFS_API, HTTPMethod.POST, HTTPStatus.OK)
    DELETE_TYPE_DEFS = API(TYPEDEFS_API, HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    DELETE_TYPE_DEF_BY_NAME = API(TYPEDEF_BY_NAME, HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)

    def __init__(self, client):
        self.client = client

    def get_all_typedefs(self, search_filter):
        return self.client.call_api(TypeDefClient.GET_ALL_TYPE_DEFS, AtlasTypesDef, search_filter.params)

    def get_all_typedef_headers(self, search_filter):
        return self.client.call_api(TypeDefClient.GET_ALL_TYPE_DEF_HEADERS, list, search_filter.params)

    def type_with_guid_exists(self, guid):
        try:
            obj = self.client.call_api(TypeDefClient.GET_TYPEDEF_BY_GUID.format_path_with_params(guid), str)

            if obj is None:
                return False
        except Exception:
            return False

        return True

    def type_with_name_exists(self, name):
        try:
            obj = self.client.call_api(TypeDefClient.GET_TYPEDEF_BY_NAME.format_path_with_params(name), str)

            if obj is None:
                return False
        except Exception:
            return False

        return True

    def get_enumdef_by_name(self, name):
        return self.__get_typedef_by_name(name, AtlasEnumDef)

    def get_enumdef_by_guid(self, guid):
        return self.__get_typedef_by_guid(guid, AtlasEntityDef)

    def get_structdef_by_name(self, name):
        return self.__get_typedef_by_name(name, AtlasStructDef)

    def get_structdef_by_guid(self, guid):
        return self.__get_typedef_by_guid(guid, AtlasStructDef)

    def get_classificationdef_by_name(self, name):
        return self.__get_typedef_by_name(name, AtlasClassificationDef)

    def get_Classificationdef_by_guid(self, guid):
        return self.__get_typedef_by_guid(guid, AtlasClassificationDef)

    def get_entitydef_by_name(self, name):
        return self.__get_typedef_by_name(name, AtlasEntityDef)

    def get_entitydef_by_guid(self, guid):
        return self.__get_typedef_by_guid(guid, AtlasEntityDef)

    def get_relationshipdef_by_name(self, name):
        return self.__get_typedef_by_name(name, AtlasRelationshipDef)

    def get_relationshipdef_by_guid(self, guid):
        return self.__get_typedef_by_guid(guid, AtlasRelationshipDef)

    def get_businessmetadatadef_by_name(self, name):
        return self.__get_typedef_by_name(name, AtlasBusinessMetadataDef)

    def get_businessmetadatadef_by_guid(self, guid):
        return self.__get_typedef_by_guid(guid, AtlasBusinessMetadataDef)

    def create_atlas_typedefs(self, types_def):
        return self.client.call_api(TypeDefClient.CREATE_TYPE_DEFS, AtlasTypesDef, None, types_def)

    def update_atlas_typedefs(self, types_def):
        return self.client.call_api(TypeDefClient.UPDATE_TYPE_DEFS, AtlasTypesDef, None, types_def)

    def delete_atlas_typedefs(self, types_def):
        return self.client.call_api(TypeDefClient.DELETE_TYPE_DEFS, None, types_def)

    def delete_type_by_name(self, type_name):
        return self.client.call_api(TypeDefClient.DELETE_TYPE_DEF_BY_NAME.format_path_with_params(type_name))

    def __get_typedef_by_name(self, name, typedef_class):
        path_type = self.__get_path_for_type(typedef_class)
        api = API(TypeDefClient.GET_BY_NAME_TEMPLATE, HTTPMethod.GET, HTTPStatus.OK)

        return self.client.call_api(api.format_path({'path_type': path_type, 'name': name}), typedef_class)

    def __get_typedef_by_guid(self, guid, typedef_class):
        path_type = self.__get_path_for_type(typedef_class)
        api = API(TypeDefClient.GET_BY_GUID_TEMPLATE, HTTPMethod.GET, HTTPStatus.OK)

        return self.client.call_api(api.format_path({'path_type': path_type, 'guid': guid}), typedef_class)

    def __get_path_for_type(self, typedef_class):
        if issubclass(AtlasEnumDef, typedef_class):
            return "enumdef"
        if issubclass(AtlasEntityDef, typedef_class):
            return "entitydef"
        if issubclass(AtlasClassificationDef, typedef_class):
            return "classificationdef"
        if issubclass(AtlasStructDef, typedef_class):
            return "structdef"
        if issubclass(AtlasRelationshipDef, typedef_class):
            return "relationshipdef"
        if issubclass(AtlasBusinessMetadataDef, typedef_class):
            return "businessmetadatadef"

        return ""



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.lineage import AtlasLineageInfo
from apache_atlas.utils import API
from apache_atlas.utils import attributes_to_params
from apache_atlas.utils import BASE_URI
from apache_atlas.utils import HTTPMethod
from apache_atlas.utils import HTTPStatus


class LineageClient:
    LINEAGE_URI = BASE_URI + "v2/lineage"
    LINEAGE_INFO = API(LINEAGE_URI, HTTPMethod.GET, HTTPStatus.OK)
    GET_LINEAGE_BY_ATTRIBUTES = API(LINEAGE_URI + "/uniqueAttribute/type/", HTTPMethod.GET, HTTPStatus.OK)

    def __init__(self, client):
        self.client = client

    def get_lineage_info(self, guid, direction, depth):
        query_params = {"direction": direction, "depth": depth}

        return self.client.call_api(LineageClient.LINEAGE_INFO.format_path_with_params(guid), AtlasLineageInfo,
                                    query_params)

    def get_lineage_info_attr(self, type_name, attributes, direction, depth):
        query_params = attributes_to_params(attributes, {"direction": direction, "depth": depth})

        return self.client.call_api(LineageClient.GET_LINEAGE_BY_ATTRIBUTES.format_path_with_params(type_name),
                                    AtlasLineageInfo, query_params)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.glossary import AtlasGlossary
from apache_atlas.model.glossary import AtlasGlossaryCategory
from apache_atlas.model.glossary import AtlasGlossaryExtInfo
from apache_atlas.model.glossary import AtlasGlossaryTerm
from apache_atlas.utils import API
from apache_atlas.utils import APPLICATION_JSON
from apache_atlas.utils import APPLICATION_OCTET_STREAM
from apache_atlas.utils import BASE_URI
from apache_atlas.utils import HTTPMethod
from apache_atlas.utils import HTTPStatus
from apache_atlas.utils import MULTIPART_FORM_DATA


class GlossaryClient:
    GLOSSARY_URI = BASE_URI + "v2/glossary"
    GLOSSARY_TERM = GLOSSARY_URI + "/term"
    GLOSSARY_TERMS = GLOSSARY_URI + "/terms"
    GLOSSARY_CATEGORY = GLOSSARY_URI + "/category"
    GLOSSARY_CATEGORIES = GLOSSARY_URI + "/categories"

    GET_ALL_GLOSSARIES = API(GLOSSARY_URI, HTTPMethod.GET, HTTPStatus.OK)
    GET_GLOSSARY_BY_GUID = API(GLOSSARY_URI + "/{glossary_guid}", HTTPMethod.GET, HTTPStatus.OK)
    GET_DETAILED_GLOSSARY = API(GLOSSARY_URI + "/{glossary_guid}/detailed", HTTPMethod.GET, HTTPStatus.OK)

    GET_GLOSSARY_TERM = API(GLOSSARY_TERM, HTTPMethod.GET, HTTPStatus.OK)
    GET_GLOSSARY_TERMS = API(GLOSSARY_URI + "/{glossary_guid}/terms", HTTPMethod.GET, HTTPStatus.OK)
    GET_GLOSSARY_TERMS_HEADERS = API(GLOSSARY_URI + "/{glossary_guid}/terms/headers", HTTPMethod.GET, HTTPStatus.OK)

    GET_GLOSSARY_CATEGORY = API(GLOSSARY_CATEGORY, HTTPMethod.GET, HTTPStatus.OK)
    GET_GLOSSARY_CATEGORIES = API(GLOSSARY_URI + "/{glossary_guid}/categories", HTTPMethod.GET, HTTPStatus.OK)
    GET_GLOSSARY_CATEGORIES_HEADERS = API(
        GLOSSARY_URI + "/{glossary_guid}/categories/headers", HTTPMethod.GET, HTTPStatus.OK)

    GET_CATEGORY_TERMS = API(GLOSSARY_CATEGORY + "/{category_guid}/terms", HTTPMethod.GET, HTTPStatus.OK)
    GET_RELATED_TERMS = API(GLOSSARY_TERMS + "/{term_guid}/related", HTTPMethod.GET, HTTPStatus.OK)
    GET_RELATED_CATEGORIES = API(GLOSSARY_CATEGORY + "/{category_guid}/related", HTTPMethod.GET, HTTPStatus.OK)
    CREATE_GLOSSARY = API(GLOSSARY_URI, HTTPMethod.POST, HTTPStatus.OK)
    CREATE_GLOSSARY_TERM = API(GLOSSARY_TERM, HTTPMethod.POST, HTTPStatus.OK)
    CREATE_GLOSSARY_TERMS = API(GLOSSARY_TERMS, HTTPMethod.POST, HTTPStatus.OK)
    CREATE_GLOSSARY_CATEGORY = API(GLOSSARY_CATEGORY, HTTPMethod.POST, HTTPStatus.OK)
    CREATE_GLOSSARY_CATEGORIES = API(GLOSSARY_CATEGORIES, HTTPMethod.POST, HTTPStatus.OK)

    UPDATE_GLOSSARY_BY_GUID = API(GLOSSARY_URI + "/{glossary_guid}", HTTPMethod.PUT, HTTPStatus.OK)
    UPDATE_PARTIAL_GLOSSARY = API(GLOSSARY_URI + "/{glossary_guid}/partial", HTTPMethod.PUT, HTTPStatus.OK)
    UPDATE_GLOSSARY_TERM = API(GLOSSARY_TERM + "/{term_guid}", HTTPMethod.PUT, HTTPStatus.OK)
    UPDATE_PARTIAL_TERM = API(GLOSSARY_TERM + "/{term_guid}/partial", HTTPMethod.PUT, HTTPStatus.OK)

    UPDATE_CATEGORY_BY_GUID = API(GLOSSARY_CATEGORY + "/{category_guid}", HTTPMethod.PUT, HTTPStatus.OK)
    UPDATE_PARTIAL_CATEGORY = API(GLOSSARY_CATEGORY + "/{category_guid}/partial", HTTPMethod.PUT, HTTPStatus.OK)

    DELETE_GLOSSARY_BY_GUID = API(GLOSSARY_URI + "/{glossary_guid}", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    DELETE_TERM_BY_GUID = API(GLOSSARY_TERM + "/{term_guid}", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)
    DELETE_CATEGORY_BY_GUID = API(GLOSSARY_CATEGORY + "/{category_guid}", HTTPMethod.DELETE, HTTPStatus.NO_CONTENT)

    GET_ENTITIES_ASSIGNED_WITH_TERM = API(
        GLOSSARY_TERMS + "/{term_guid}/assignedEntities", HTTPMethod.GET, HTTPStatus.OK)
    ASSIGN_TERM_TO_ENTITIES = API(
        GLOSSARY_TERMS + "/{term_guid}/assignedEntities", HTTPMethod.POST, HTTPStatus.NO_CONTENT)
    DISASSOCIATE_TERM_FROM_ENTITIES = API(
        GLOSSARY_TERMS + "/{term_guid}/assignedEntities", HTTPMethod.PUT, HTTPStatus.NO_CONTENT)

    GET_IMPORT_GLOSSARY_TEMPLATE = API(GLOSSARY_URI + "/import/template", HTTPMethod.GET,
                                       HTTPStatus.OK, APPLICATION_JSON, APPLICATION_OCTET_STREAM)
    IMPORT_GLOSSARY = API(GLOSSARY_URI + "/import", HTTPMethod.POST,
                          HTTPStatus.OK, MULTIPART_FORM_DATA, APPLICATION_JSON)

    QUERY = "query"
    LIMIT = "limit"
    OFFSET = "offset"
    STATUS = "Status"

    DEFAULT_LIMIT = -1
    DEFAULT_OFFSET = 0
    DEFAULT_SORT = "ASC"

    def __init__(self, client):
        self.client = client

    def get_all_glossaries(self, sort_by_attribute=DEFAULT_SORT, limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {"sort": sort_by_attribute, GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset}

        return self.client.call_api(GlossaryClient.GET_ALL_GLOSSARIES, list, query_params)

    def get_glossary_by_guid(self, glossary_guid):
        return self.client.call_api(GlossaryClient.GET_GLOSSARY_BY_GUID.format_path({'glossary_guid': glossary_guid}),
                                    AtlasGlossary)

    def get_glossary_ext_info(self, glossary_guid):
        return self.client.call_api(GlossaryClient.GET_DETAILED_GLOSSARY.format_path({'glossary_guid': glossary_guid}),
                                    AtlasGlossaryExtInfo)

    def get_glossary_term(self, term_guid):
        return self.client.call_api(GlossaryClient.GET_GLOSSARY_TERM.format_path_with_params(term_guid),
                                    AtlasGlossaryTerm)

    def get_glossary_terms(self, glossary_guid, sort_by_attribute=DEFAULT_SORT,
                           limit=DEFAULT_LIMIT, offset=DEFAULT_OFFSET):
        query_params = {"glossaryGuid": glossary_guid, GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset,
                        "sort": sort_by_attribute}

        return self.client.call_api(GlossaryClient.GET_GLOSSARY_TERMS.format_path({'glossary_guid': glossary_guid}),
                                    list, query_params)

    def get_glossary_term_headers(self, glossary_guid, sort_by_attribute, limit, offset):
        query_params = {"glossaryGuid": glossary_guid, GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset,
                        "sort": sort_by_attribute}

        return self.client.call_api(
            GlossaryClient.GET_GLOSSARY_TERMS_HEADERS.format_path({'glossary_guid': glossary_guid}),
            list, query_params)

    def get_glossary_category(self, category_guid):
        return self.client.call_api(GlossaryClient.GET_GLOSSARY_CATEGORY.format_path_with_params(category_guid),
                                    AtlasGlossaryCategory)

    def get_glossary_categories(self, glossary_guid, sort_by_attribute, limit, offset):
        query_params = {"glossaryGuid": glossary_guid, GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset,
                        "sort": sort_by_attribute}

        return self.client.call_api(
            GlossaryClient.GET_GLOSSARY_CATEGORIES.format_path({'glossary_guid': glossary_guid}),
            list, query_params)

    def get_glossary_category_headers(self, glossary_guid, sort_by_attribute, limit, offset):
        query_params = {"glossaryGuid": glossary_guid, GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset,
                        "sort": sort_by_attribute}

        return self.client.call_api(
            GlossaryClient.GET_GLOSSARY_CATEGORIES_HEADERS.format_path({'glossary_guid': glossary_guid}),
            list, query_params)

    def get_category_terms(self, category_guid, sort_by_attribute, limit, offset):
        query_params = {"categoryGuid": category_guid, GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset,
                        "sort": sort_by_attribute}

        return self.client.call_api(GlossaryClient.GET_CATEGORY_TERMS.format_path({'category_guid': category_guid}),
                                    list, query_params)

    def get_related_terms(self, term_guid, sort_by_attribute, limit, offset):
        query_params = {"termGuid": term_guid, GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset,
                        "sort": sort_by_attribute}

        return self.client.call_api(GlossaryClient.GET_RELATED_TERMS.format_path({'term_guid': term_guid}), dict,
                                    query_params)

    def get_related_categories(self, category_guid, sort_by_attribute, limit, offset):
        query_params = {GlossaryClient.LIMIT: limit, GlossaryClient.OFFSET: offset, "sort": sort_by_attribute}

        return self.client.call_api(GlossaryClient.GET_RELATED_CATEGORIES.format_path({'category_guid': category_guid}),
                                    dict, query_params)

    def create_glossary(self, glossary):
        return self.client.call_api(GlossaryClient.CREATE_GLOSSARY, AtlasGlossary, None, glossary)

    def create_glossary_term(self, glossary_term):
        return self.client.call_api(GlossaryClient.CREATE_GLOSSARY_TERM, AtlasGlossaryTerm, None, glossary_term)

    def create_glossary_terms(self, glossary_terms):
        return self.client.call_api(GlossaryClient.CREATE_GLOSSARY_TERMS, list, None, glossary_terms)

    def create_glossary_category(self, glossary_category):
        return self.client.call_api(GlossaryClient.CREATE_GLOSSARY_CATEGORY, AtlasGlossaryCategory, None,
                                    glossary_category)

    def create_glossary_categories(self, glossary_categories):
        return self.client.call_api(GlossaryClient.CREATE_GLOSSARY_CATEGORIES, list, glossary_categories)

    def update_glossary_by_guid(self, glossary_guid, updated_glossary):
        return self.client.call_api(
            GlossaryClient.UPDATE_GLOSSARY_BY_GUID.format_path({'glossary_guid': glossary_guid}),
            AtlasGlossary, None, updated_glossary)

    def partial_update_glossary_by_guid(self, glossary_guid, attributes):
        return self.client.call_api(
            GlossaryClient.UPDATE_PARTIAL_GLOSSARY.format_path({'glossary_guid': glossary_guid}),
            AtlasGlossary, attributes)

    def update_glossary_term_by_guid(self, term_guid, glossary_term):
        return self.client.call_api(GlossaryClient.UPDATE_GLOSSARY_TERM.format_path({'term_guid': term_guid}),
                                    AtlasGlossaryTerm, None, glossary_term)

    def partial_update_term_by_guid(self, term_guid, attributes):
        return self.client.call_api(GlossaryClient.UPDATE_PARTIAL_TERM.format_path({'term_guid': term_guid}),
                                    AtlasGlossaryTerm, attributes)

    def update_glossary_category_by_guid(self, category_guid, glossary_category):
        return self.client.call_api(
            GlossaryClient.UPDATE_CATEGORY_BY_GUID.format_path({'category_guid': category_guid}),
            AtlasGlossaryCategory, glossary_category)

    def partial_update_category_by_guid(self, category_guid, attributes):
        return self.client.call_api(
            GlossaryClient.UPDATE_PARTIAL_CATEGORY.format_path({'category_guid': category_guid}),
            AtlasGlossaryCategory, None, attributes)

    def delete_glossary_by_guid(self, glossary_guid):
        return self.client.call_api(
            GlossaryClient.DELETE_GLOSSARY_BY_GUID.format_path({'glossary_guid': glossary_guid}))

    def delete_glossary_term_by_guid(self, term_guid):
        return self.client.call_api(GlossaryClient.DELETE_TERM_BY_GUID.format_path({'term_guid': term_guid}))

    def delete_glossary_category_by_guid(self, category_guid):
        return self.client.call_api(
            GlossaryClient.DELETE_CATEGORY_BY_GUID.format_path({'category_guid': category_guid}))

    def get_entities_assigned_with_term(self, term_guid, sort_by_attribute, limit, offset):
        query_params = {"termGuid": term_guid, GlossaryClient.LIMIT: limit,
                        GlossaryClient.OFFSET: offset, "sort": sort_by_attribute}

        return self.client.call_api(
            GlossaryClient.GET_ENTITIES_ASSIGNED_WITH_TERM.format_path({'term_guid': term_guid}), list, query_params)

    def assign_term_to_entities(self, term_guid, related_object_ids):
        return self.client.call_api(GlossaryClient.ASSIGN_TERM_TO_ENTITIES.format_path({'term_guid': term_guid}),
                                    None, None, related_object_ids)

    def disassociate_term_from_entities(self, term_guid, related_object_ids):
        return self.client.call_api(
            GlossaryClient.DISASSOCIATE_TERM_FROM_ENTITIES.format_path({'term_guid': term_guid}),
            None, None, related_object_ids)



#!/usr/bin/env/python
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from apache_atlas.model.instance import AtlasClassification
from apache_atlas.model.instance import AtlasEntityHeader
from apache_atlas.model.instance import AtlasObjectId
from apache_atlas.model.instance import AtlasStruct
from apache_atlas.model.misc import AtlasBase
from apache_atlas.utils import type_coerce
from apache_atlas.utils import type_coerce_dict
from apache_atlas.utils import type_coerce_list


class AtlasRelationship(AtlasStruct):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasStruct.__init__(self, attrs)

        self.guid = attrs.get('guid')
        self.homeId = attrs.get('homeId')
        self.provenanceType = attrs.get('provenanceType')
        self.end1 = attrs.get('end1')
        self.end2 = attrs.get('end2')
        self.label = attrs.get('label')
        self.propagateTags = attrs.get('propagateTags')
        self.status = attrs.get('status')
        self.createdBy = attrs.get('createdBy')
        self.updatedBy = attrs.get('updatedBy')
        self.createTime = attrs.get('createTime')
        self.updateTime = attrs.get('updateTime')
        self.version = attrs.get('version')
        self.propagatedClassifications = attrs.get('propagatedClassifications')
        self.blockedPropagatedClassifications = attrs.get('blockedPropagatedClassifications')

    def type_coerce_attrs(self):
        super(AtlasRelationship, self).type_coerce_attrs()

        self.end1 = type_coerce(self.end1, AtlasObjectId)
        self.end2 = type_coerce(self.end2, AtlasObjectId)
        self.propagatedClassifications = type_coerce_list(self.propagatedClassifications, AtlasClassification)
        self.blockedPropagatedClassifications = type_coerce_list(
            self.blockedPropagatedClassifications, AtlasClassification)


class AtlasRelationshipWithExtInfo(AtlasBase):
    def __init__(self, attrs=None):
        attrs = attrs or {}

        AtlasBase.__init__(self, attrs)

        self.relationship = attrs.get('relationship')
        self.referredEntities = attrs.get('referredEntities')

    def type_coerce_attrs(self):
        super(AtlasBase, self).type_coerce_attrs()

        self.relationship = type_coerce(self.relationship, AtlasRelationship)
        self.referredEntities = type_coerce_dict(self.referredEntities, AtlasEntityHeader)
